# -*- coding: utf-8 -*-
from posixpath import split
import sys
import os
from binhex import openrsrc
import re
import multiprocessing

RED   = "\033[1;31m"
BLUE  = "\033[1;34m"
CYAN  = "\033[1;36m"
GREEN = "\033[0;32m"
RESET = "\033[0;0m"
BOLD    = "\033[;1m"
REVERSE = "\033[;7m"

def print_help():
    print("Options are:")
    print("  --help, -h show options")
    print("  == Mandatory ==")
    print("    --target, -t = <posix|xenomai> define the target")
    print("  == Optional ==")
    print("    --unit, -u = <x><us|ms|s> set the time unit (second by default)")
    print("    --debug, -d = <inline|scn> print debug information during the execution, scn format can be import in play from Tina toolbox ")
    print("    --enable, -e = <yes> turn on initially debug enable flag (turned off by default)")
    print("    --lttng, -l = <yes> add lltng tracepoints")
    print("    --gdb, -g = <yes> add -g option flag to generate debug information to be used by GDB debugger")
    print("    --hippolib, -h = <path> specify the path (realtive or absolute) of the hippo library")
    print("    --library, -lib = <l1,l2,l3> list additional libraries")
    print("    --affinitiy, -a = <yes> allocate engine on cpu0 and other threads on other core")
    print("  == Optional with Posix ==")
    print("    --nanosleep, -n = <yes> using clock_nanosleep instead of timer")
    print("    --sched_fifo, -f = <yes> enable SCHED_FIFO (need root privilege to execute)")
    

#############################
# COMPILATION VARIABLES     #
HIPPOLIB="../../lib/hippo"
XENOMAILIB="/usr/xenomai/lib"
MERCURYLIB="/usr/xenomai/include/mercury"
ALCHEMYLIB="/usr/xenomai/include/alchemy"
INCLIB="/usr/xenomai/include"

# default compiler to gcc
CC="gcc"

# default optimizations flags
OPTFLAGS="-O2"

CFLAGS =" -DNDEBUG=1 "
LFLAGS = ""

#############################
# OPTION VARIABLES          #

project = ""

dic = {"--target":"target",
       "-t":"target",
       "--unit":"unit",
       "-u":"unit",
       "--debug":"debug",
       "-d":"debug",
       "--enable":"enable",
       "-e":"enable",
       "--lttng":"lttng",
       "-l":"lttng",
       "--gdb":"gdb",
       "-g":"gdb",
       "--hippolib":"hippolib",
       "-h":"hippolib",
       "--library":"library",
       "-lib":"library",
       "--affinitiy":"affinity",
       "-a":"affinity",
       "--nanosleep":"nanosleep",
       "-n":"nanosleep",
       "--sched_fifo":"schedfifo",
       "-f":"schedfifo"}

options = {}
list_target = ["posix", "xenomai"]
list_unit = ["us", "ms", "s"]
list_debug = ["inline", "scn", None]
list_enable = ["yes", None]
list_lttng = ["yes", None]
list_gdb = ["yes", None]

make = ""

#############################
# CHECK BASIC OPTION        #

if (len(sys.argv)==1) or (sys.argv[1] == "--help") or (sys.argv[1] == "-h"):
    print_help()
    exit()

project = sys.argv[1]
try:
    os.listdir(os.getcwd()).index(project+".hippo")
except:
    print("The repository {0}.hippo does not exist.".format(sys.argv[1]))
    exit()

# parse options
for arg in sys.argv[2:]:
    s_arg = arg.split("=")
    try :
        dic[s_arg[0]]
    except:
        print("{0}error:{1} {2} bad option".format(RED, RESET, s_arg[0]))
        exit()
    options[dic[s_arg[0]]] = s_arg[1]

# check options
try:
    options["target"]
except:
    print("{0}error:{1} the target option is mandatory".format(RED, RESET))
    exit()
try:
    options["unit"]
    options["unit"]=re.findall(r'([0-9]*)(.*)', options["unit"])
except:
    print("{0}warning:{1} default unit is second".format(CYAN, RESET))
    options["unit"] = "s"
try:
    options["debug"]
except:
    options["debug"] = None
try:
    options["enable"]
except:
    options["enable"] = None
try:
    options["lttng"]
except:
    options["lttng"] = None
try:
    options["gdb"]
except:
    options["gdb"] = None
try:
    options["hippolib"]
except:
    options["hippolib"] = None
try:
    options["library"]
except:
    options["library"] = None
try:
    options["affinity"]
except:
    options["affinity"] = None
try:
    options["nanosleep"]
except:
    options["nanosleep"] = None
try:
    options["schedfifo"]
except:
    options["schedfifo"] = None

# check optional
try:
    list_target.index(options["target"])
except:
    print("{0}error:{1} the target value ({2}) is not valid".format(RED, RESET, options["target"]))
    exit()

try:
    list_unit.index(options["unit"][0][1])
except:
    print("{0}warning:{1} the unit value ({2}) is not valid".format(CYAN, RESET, options["unit"]))
    options["unit"]=[("1","s")]

try:
    list_debug.index(options["debug"])
except:
    print("{0}warning:{1} the debug value ({2}) is not valid".format(CYAN, RESET, options["debug"]))
    options["debug"]= None

try:
    list_enable.index(options["enable"])
except:
    print("{0}warning:{1} the enable value ({2}) is not valid".format(CYAN, RESET, options["enable"]))
    options["enable"]= None

try:
    list_lttng.index(options["lttng"])
except:
    print("{0}warning:{1} the lttng value ({2}) is not valid".format(CYAN, RESET, options["lttng"]))
    options["lttng"] = None

try:
    list_gdb.index(options["gdb"])
except:
    print("{0}warning:{1} the gdb value ({2}) is not valid".format(CYAN, RESET, options["gdb"]))
    options["gdb"] = None

# Begin to process the makefile
if options["library"]:
    for l in options["library"].split(','):
        LFLAGS+= " -l"+l

if options["hippolib"]:
    HIPPOLIB=options["hippolib"]

if options["target"] == "xenomai":
    CFLAGS += " -D XENOMAI"
elif options["target"] == "posix":
    CFLAGS += " -D POSIX"

if options["debug"] == "inline":
    CFLAGS += " -D_TRACE_ -D_DEBUG_ON_"
if options["debug"] == "scn":
    CFLAGS += " -D_SCN_"

if options["enable"] == "yes":
    CFLAGS += " -D_DEBUG_ON_"

if options["unit"][0][1] == "us":
    CFLAGS += " -D_TICK_UNIT_={0}".format(int(options["unit"][0][0])*1000)
elif options["unit"][0][1] == "ms":
    CFLAGS += " -D_TICK_UNIT_={0}".format(int(options["unit"][0][0])*1000000)
elif options["unit"][0][1] == "s":
    CFLAGS += " -D_TICK_UNIT_={0}".format(int(options["unit"][0][0])*1000000000)

if options["gdb"] == "yes":
    OPTFLAGS = "-O0 -g"

if options["affinity"] == "yes":
    if (multiprocessing.cpu_count() == 1):
        print("{0}warning:{1} you've got only one cpu,the affinity option is not available".format(CYAN, RESET, options["gdb"]))
    else:
        OPTFLAGS += " -D AFFINITY"

if options["nanosleep"] == "yes":
    if not (options["target"] == "posix"):
        print("{0}warning:{1} nanosleep can only be used with posix".format(CYAN, RESET, options["gdb"]))
    else:
        OPTFLAGS += " -D NANOSLEEP"

if options["schedfifo"] == "yes":
    if not (options["target"] == "posix"):
        print("{0}warning:{1} SCHED_FIFO can only be used with posix".format(CYAN, RESET, options["gdb"]))
    else:
        OPTFLAGS += " -D ACTIVATE_SCHED_FIFO"


if options["lttng"] == "yes":
    CFLAGS += " -D_LTTNG_"
    LFLAGS += " -llttng-ust -ldl"

#######
make = "all: " + (project) + "\n"
make = (project) + ":\n"

if options["lttng"] == "yes":
    make += "\t" + CC +" -c -I " + HIPPOLIB + " " + HIPPOLIB + "/hippo-tp.c"
    make += "\n"

if options["target"] == "xenomai":
    make += "\t" +CC + " -c " + CFLAGS + " " + OPTFLAGS + " -I " + HIPPOLIB +" -I " + XENOMAILIB +\
            " -I " + MERCURYLIB + " -I " + INCLIB + " -D_GNU_SOURCE -D_REENTRANT -D__MERCURY_ -I "+\
            ALCHEMYLIB + " -w -I ./" + project + ".hippo " + HIPPOLIB + "/hippo.c " + project +\
            ".hippo/*.c ./main.c"
    make += "\n"
    make += "\t" + CC + " -o " + project + " ./*.o " + XENOMAILIB + "/xenomai/bootstrap.o " + \
            "-ldl -Wl,--wrap=main -Wl,--dynamic-list="+ XENOMAILIB + "/dynlist.ld -L " + XENOMAILIB +\
            " -lalchemy -lcopperplate -lmercury -lpthread -lrt -ldl -Wl,-R " + XENOMAILIB + " " + LFLAGS
    make += "\n"
elif options["target"] == "posix":
    make += "\t" + CC + " -c " + (CFLAGS) + " " + (OPTFLAGS) + " -I " + (HIPPOLIB) + \
            " -fmessage-length=0 -I ./" + (project) + ".hippo "+ (HIPPOLIB) +"/hippo.c "+  (project) +".hippo/*.c ./main.c"
    make += "\n"
    make += "\t" + (CC) + " -o " + (project) + " ./*.o -lpthread -lrt" + " " + LFLAGS
    make += "\n"

make += "\trm *.o"
make += "\n"
make += "clean:\n"
make += "\trm " + (project) + " | true"
make += "\n"
make += "new: clean\n"
make += "\t$(MAKE)"
make += "\n"

# write makefile
f = open("Makefile", "w")
f.write(make)
f.close()

print("** The Makefile is generated\n** Make to compile")
